<?php
/*
Plugin Name: FlattrButton standalone
Plugin URI: http://blog.mmn-o.se/dev/flattrbutton/
Description: A plugin that lets you add a <a href="http://flattr.com/">Flattr</a> button with configurable URL in the sidebar and/or after posts. Code has been to some degree copied from Flattr.com and Donncha O Caoimh's Flattr WP plugins.
Version: 0.0.3
Author: Mikael "MMN-o" Nordfeldth
Author URI: http://blog.mmn-o.se/
*/

// Released under the GNU Affero GPL
// http://www.gnu.org/licenses/agpl-3.0.html
//

if (!defined('FLATTR_WP_SCRIPT')) {
	define('FLATTR_WP_SCRIPT',  'http://api.flattr.com/button/load.js');
}

class FlattrButton {
	private $conditionals = array(
			'is_home' => 'The front page',
			'is_single' => 'Single posts',
			'is_page' => 'Single pages',
			'is_category' => 'Posts in category archive',
			'is_tag' => 'Posts in tag archive',
			'is_date' => 'Posts in date archive',
			'is_author' => 'Posts in author archive',
			'is_search' => 'Posts in search result',
			'is_feed' => 'Posts in RSS feed'
		);
	private $cats = array(
			'text' => 'Text',
			'images' => 'Images',
			'audio' => 'Audio',
			'video' => 'Video',
			'software' => 'Software',
			'rest' => 'The rest'
		);
	private $opts = array(
			'url' => 'url',
			'tle' => 'title',
			'dsc' => 'description',
			'tag' => 'tag',
			'btn' => 'button',
			'lng' => 'language',
			'uid' => 'uid',
			'cat' => 'cat'
		);
	private $lngs = array(
			'sq_AL' => 'Albanian',
			'ar_DZ' => 'Arabic',
			'be_BY' => 'Belarusian',
			'bg_BG' => 'Bulgarian',
			'ca_ES' => 'Catalan',
			'zh_CN' => 'Chinese',
			'hr_HR' => 'Croatian',
			'cs_CZ' => 'Czech',
			'da_DK' => 'Danish',
			'nl_NL' => 'Dutch',
			'en_GB' => 'English',
			'et_EE' => 'Estonian',
			'fi_FI' => 'Finnish',
			'fr_FR' => 'French',
			'de_DE' => 'German',
			'el_GR' => 'Greek',
			'iw_IL' => 'Hebrew',
			'hi_IN' => 'Hindi',
			'hu_HU' => 'Hungarian',
			'is_IS' => 'Icelandic',
			'in_ID' => 'Indonesian',
			'ga_IE' => 'Irish',
			'it_IT' => 'Italian',
			'ja_JP' => 'Japanese',
			'ko_KR' => 'Korean',
			'lv_LV' => 'Latvian',
			'lt_LT' => 'Lithuanian',
			'mk_MK' => 'Macedonian',
			'ms_MY' => 'Malay',
			'mt_MT' => 'Maltese',
			'no_NO' => 'Norwegian',
			'pl_PL' => 'Polish',
			'pt_PT' => 'Portuguese',
			'ro_RO' => 'Romanian',
			'ru_RU' => 'Russian',
			'sr_RS' => 'Serbian',
			'sk_SK' => 'Slovak',
			'sl_SI' => 'Slovenian',
			'es_ES' => 'Spanish',
			'sv_SE' => 'Swedish',
			'th_TH' => 'Thai',
			'tr_TR' => 'Turkish',
			'uk_UA' => 'Ukrainian',
			'vi_VN' => 'Vietnamese'
		);

	/* Default settings */
	private $flattr_btn = '';
	private $flattr_cat = '';
	private $flattr_dsc = '';
	private $flattr_lng = '';
	private $flattr_tag = '';
	private $flattr_tle = '';
	private $flattr_uid = '';
	private $flattr_url = '';
	
	/* When should the button be generated? */
	private $conditions = array();

	function __construct() {
		$this->settings = array(
			'title' => __("Support my Thing!"),
		);
		// not included in options
		$this->version = 'undef';
		
		$this->set_conditions(get_option('flattrbutton_conditions', array()));
		$this->update_meta();
	}
	private function update_meta( $id=NULL ) {
		if (!empty($id)) {
			$this->update_post_meta( $id );
		} else {
			$this->set_uid(get_option('flattrbutton_option_uid'));
			$this->set_url(site_url());
			$this->set_tle(get_bloginfo('name'));
			$this->set_dsc(get_bloginfo('description'));
			$this->set_btn('normal');
			$this->set_tag('');
			$this->set_lng(get_option('flattrbutton_option_lng'));
			$this->set_cat(get_option('flattrbutton_option_cat'));
		}
	}
	private function update_post_meta( $id ) {
		$mytags = array();
		foreach(get_the_tags($id) as $tag) {
			$mytags[] = htmlspecialchars($tag->name, ENT_QUOTES);
		}

		$this->set_url(get_permalink($id));
		$this->set_tle(get_the_title($id));
		$this->set_dsc(get_the_title($id)); //because this loops: get_the_excerpt($id));
		$this->set_tag(implode(',', $mytags));
		$this->set_btn('compact');
		$_ = get_post_meta($id, '_flattr_post_language', true);
		$this->set_lng(!empty($_) ? $_ : $this->get_lng());
		$_ = get_post_meta($id, '_flattr_post_uid', true);
		$this->set_uid(!empty($_) ? $_ : $this->get_uid());
		$_ = get_post_meta($id, '_flattr_post_category', true);
		$this->set_cat(!empty($_) ? $_ : $this->get_cat());
	}

	function get_categories() {
		return $this->cats;
	}
	function get_conditionals() {
		return $this->conditionals;
	}
	function get_condition($condition) {
		switch ($status = $this->conditions[$condition]) {
		case '':
			$status = 'off';
		case 'on':
		case 'off':
			break;
		default:
			$status = 'on';
		}
		return $status;
	}
	function get_conditions() {
		return $this->conditions;
	}
	function get_languages() {
		return $this->lngs;
	}
	function get_options() {
		return $this->opts;
	}

	function get_btn() {
		return $this->flattr_btn;
	}
	function get_cat() {
		return $this->flattr_cat;
	}
	function get_dsc() {
		return $this->flattr_dsc;
	}
	function get_lng() {
		return $this->flattr_lng;
	}
	function get_tag() {
		return $this->flattr_tag;
	}
	function get_tle() {
		return $this->flattr_tle;
	}
	function get_uid() {
		return $this->flattr_uid;
	}
	function get_url() {
		return $this->flattr_url;
	}

	function set_cat( $cat ) {
		if (!array_key_exists($cat, $this->get_categories())) {
			return FALSE;
		}
		return $this->flattr_cat = $cat;
	}
	function set_conditions($conditions) {
	 	$new = array_intersect_key($conditions, $this->get_conditionals());
		foreach ($new as $condition=>$val) {
			if ($val != 'on') {
				unset($new[$condition]);
			}
		}
		return $this->conditions = $new;
	}
	private function set_dsc($dsc) {
		$this->flattr_dsc = $dsc;
	}
	function set_lng($lng) {
		return $this->flattr_lng = (!empty($lng) ? $lng : $this->get_lng());
	}
	private function set_tag($tag) {
		$this->flattr_tag = $tag;
	}
	private function set_tle($tle) {
		$this->flattr_tle = $tle;
	}
	function set_uid( $uid ) {
		return $this->flattr_uid = (!empty($uid) ? 0+$uid : 0);
	}
	private function set_url($url) {
		/* TODO: Add verification */
		$this->flattr_url = $url;
	}
	private function set_btn($btn) {
		switch ($btn) {
		case 'normal':
		case 'compact':
			$this->flattr_btn = $btn;
			break;
		default:
			$this->flattr_btn = 'normal';
		}
	}

	function generate($id=NULL) {
		$this->update_meta($id);
		$htmlid = ($id ? "flattrbutton-$id" : "flattrbutton");

		$html = '<div id="'.$htmlid.'" class="flattrbutton"><script type="text/javascript">'."\n";
		foreach($this->opts as $key=>$val) {
			$html .= "\tvar " . htmlspecialchars("flattr_$key") . ' = "' . htmlspecialchars(str_replace(array("\r","\n"),array('\r','\n'),$this->{"get_$key"}())) . '";'."\n";
		}
		$html .= '</script><script src="' . FLATTR_WP_SCRIPT . '" type="text/javascript"></script></div>';
		return $html;
	}
	function the_flattrbutton($id=NULL) {
		echo $this->generate($id);
	}
	function the_widget() {
		$this->the_flattrbutton();
	}

	private function guess_post_cat($id) {
		$post = get_post($id);
		$post_mime_type = substr($post->post_mime_type, 0, 5);
		switch ($post->post_mime_type) {
		case 'image':
		case 'video':
			$post_type = $post_mime_type;
			break;
		default:
			$post_type = 'text';
		}
		return $post_type;
	}

	function get_flattr_option($option, $id=NULL) {
		$get = NULL;
		
		switch ($option) {
		case 'lng':	case 'cat':	case 'uid':
		case 'tle':	case 'dsc':	case 'btn':
			$get = "flattr_$option";
			break;
		default:
			throw new Exception('bad FlattrButton option');
		}
		return (empty($id)
			? $this->{$get}
			: get_post_meta($id, $get, true));
	}

	/* Hooks */
		/* Got this code from the Sociable WP plugin */
	function display_hook($content='') {
		global $post;
		$html = $this->generate($post->ID);
		$conditions = $this->get_conditions();
		if ((is_home()     and $conditions['is_home']) or
		    (is_single()   and $conditions['is_single']) or
		    (is_page()     and $conditions['is_page']) or
		    (is_category() and $conditions['is_category']) or
			(is_tag() 	   and $conditions['is_tag']) or
		    (is_date()     and $conditions['is_date']) or
			(is_author()   and $conditions['is_author']) or
		    (is_search()   and $conditions['is_search'])) {
			$content .= $html;
		} elseif ((is_feed() and $conditions['is_feed'])) {
			$content .= strip_tags($html,"<a><img>") . "<br/><br/>";
		}
		return $content;
	}

	function inner_meta_box () {
		global $post;
		$flattr_lng = $this->get_lng();
		$flattr_cat = $this->get_cat();

?>
	Flattr box
<?php
	}

	function save_post( $id ) {
		if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) {
			return $id;
		}

		if ( !current_user_can('edit_post', $id) ) {
			return $id;
		}

		foreach (array('lng', 'cat') as $key) {
			add_post_meta($id, '_flattrbutton_post_'.$this->opts[$cat], $_POST['flattrbutton_post_'.$this->opts[$cat]], true)
				or update_post_meta($id, '_flattrbutton_post_'.$this->opts[$cat], $_POST['flattrbutton_post_'.$this->opts[$cat]]);
			add_post_meta($id, '_flattr_post_'.$this->opts[$cat], $_POST['flattr_post_'.$this->opts[$cat]], true)
				or update_post_meta($id, '_flattr_post_'.$this->opts[$cat], $_POST['flattr_post_'.$this->opts[$cat]]);
		}
		return true;
	}


	function flattrbutton_control() {
		if ($_POST['flattrbutton-submit']) {
			$this->settings['widget_title'] = htmlspecialchars($_POST['flattrbutton-widget-title']);
			if($this->settings['html']) { 
				$this->settings['text'] = $_POST['flattrbutton-widget-text'];
			} else {
				$this->settings['text'] = htmlspecialchars($_POST['flattrbutton-widget-text']);
			}
			$this->settings['above'] = $_POST['flattrbutton-above'];
			$this->settings['compact'] = $_POST['flattrbutton-widget-compact'];
			$this->settings['html'] = $_POST['flattrbutton-html'];

			update_option('flattrbutton', $this->settings);
		}
?>
	<input type="hidden" id="flattrwidget-submit" name="flattrwidget-submit" value="1" />
	<ul><li>
		<label for="flattrbutton-widget-title"><?php _e('Widget title:'); ?></label>
		<input class="widefat" type="text" id="flattrwidget-title" name="flattrwidget-title" value="<?php echo $options['title'];?>" />
	</li><li>
		<label for="flattrwidget-text">Text: </label>
		<textarea class="widefat" type="text" id="flattrwidget-text" name="flattrwidget-text"><?php echo stripslashes($options['text']);?></textarea>
	</li><li>
		<input type="checkbox" id="flattrwidget-above" name="flattrwidget-above"<?php if ($options['above']) { echo " checked"; } ?> />
		<label for="flattrwidget-above">Check to display the text above the Flattr button. (leave unchecked to display below)</label>
	</li><li>
		<input type="checkbox" id="flattrwidget-html" name="flattrwidget-html"<?php if ($options['html']) { echo " checked"; } ?> />
		<label for="flattrwidget-html">Check to allow HTML in text.</label>
	</li><li>
		<input type="checkbox" id="flattrwidget-compact" name="flattrwidget-compact"<?php if ($options['compact']) { echo " checked"; } ?> />
		<label for="flattrwidget-compact">Check to use compact style Flattr button.</label>
	</li><li>
<?php		if (!$this->get_uid()) : ?>
	You probably need a <a href="https://flattr.com/register">Flattr uid</a> to enjoy this plugin.
<?php		endif; ?>
<?php
	}


	/* Admin stuff */

	function admin_options() {
?>
	<div>
		<h2>FlattrButton Settings</h2>
		<form method="post" action="options.php">
<?php
		settings_fields('flattrbutton_options');
		do_settings_sections('flattrbutton');
?>
			<input name="submit" type="submit" value="<?php esc_attr_e('Save Changes'); ?>" />
		</form>
	</div>
<?php
	}
	function main_text() {
		echo '<p>You will probably <a href="https://flattr.com/register">need a Flattr account</a> to enjoy this plugin.</p>';
	}
	function display_text() {
		echo "<p>If you don't check these, you may use <code>&lt;?php the_flattrbutton(); ?&gt;</code> in your themes.</p>";
	}
	function flattr_uid() {
?>
			<input name="flattrbutton_option_uid" type="text" value="<?php echo get_option('flattrbutton_option_uid'); ?>" />
<?php
	}
	function flattr_cat() {
?>
			<select name="flattrbutton_option_cat">
<?php		foreach ( $this->get_categories() as $key=>$val ) : ?>
				<option value="<?php echo $key?>"<?php echo ($this->get_cat() == $key) ? ' selected="selected"' : ''; ?>><?php echo $val?></option>
<?php		endforeach; ?>
			</select>
<?php
	}
	function flattr_lng() {
?>
			<select name="flattrbutton_option_lng">
<?php		foreach ( $this->get_languages() as $languageCode => $language ) : ?>
				<option value="<?php echo $languageCode ?>"<?php echo $languageCode == $this->get_lng() ? ' selected' : ''; ?>><?php echo $language ?></option>
<?php		endforeach; ?>
			</select>
<?php
	}
	function display_conditions() {
?>
			<ul id="flattrbutton_conditions">
<?php		foreach ( $this->get_conditionals() as $condition=>$description ) : ?>
			<li><input <?php if ($this->get_condition($condition) == 'on') { echo ' checked="checked"'; } ?> type="checkbox" id="flattrbutton_condition_<?php echo $condition; ?>" name="flattrbutton_conditions[<?php echo $condition; ?>]" value="on" /><label for="flattrbutton_condition_<?php echo $condition; ?>"><?php echo htmlspecialchars($description); ?></label></li>
<?php		endforeach; ?>
			</ul>
<?php
	}

/* End of FlattrButton class */
}

function flattrbutton_init() {
	global $flattrbutton;
	$flattrbutton = new FlattrButton;
	if (!$flattrbutton->get_flattr_option('uid')) {
		return false;
	}
	if (!function_exists('the_flattrbutton')) {
		function the_flattrbutton() {
			global $flattrbutton;
			$flattrbutton->the_flattrbutton();
		}
		function the_flattr_permalink() {
			return the_flattrbutton();
		}
	} else {
		throw new Exception ('the_flattrbutton() exists already. Colliding plugin?');
	}

	add_filter('the_content', array($flattrbutton, 'display_hook'));
}

function flattrbutton_widget_init() {
	function register_flattrbutton_widget($args) {
		global $flattrbutton;
		extract($args);
		echo $before_widget . $before_title . $title . $after_title;
		
		$flattrbutton->the_widget();
		echo $after_widget;
	}
	wp_register_sidebar_widget('flattrbutton', 'FlattrButton', 'register_flattrbutton_widget');

}


add_action('init', 'flattrbutton_init');
add_action('widgets_init', 'flattrbutton_widget_init');

if (is_admin()) :
	add_action('admin_menu', 'flattrbutton_add_meta_box');
	add_action('admin_menu', 'flattrbutton_admin_menu');
	add_action('admin_init', 'flattrbutton_admin_init' );
	add_action('save_post', array($flattrbutton, 'save_post'));
	register_widget_control ( 'FlattrButton Widget', flattrbutton_widget_control );


	function flattrbutton_admin_menu() {
		global $flattrbutton;
		add_options_page('FlattrButton', 'FlattrButton', 'manage_options', basename(__FILE__, '.php'), array($flattrbutton, 'admin_options'));
	}
	
	function flattrbutton_admin_init() {
		global $flattrbutton;
		add_settings_section('flattrbutton_main', 'Main Settings', array($flattrbutton, 'main_text'), 'flattrbutton');
		add_settings_field('flattrbutton_option_uid', 'Flattr UID', array($flattrbutton, 'flattr_uid'), 'flattrbutton', 'flattrbutton_main');
		add_settings_field('flattrbutton_option_cat', 'Default category', array($flattrbutton, 'flattr_cat'), 'flattrbutton', 'flattrbutton_main');
		add_settings_field('flattrbutton_option_lng', 'Default language', array($flattrbutton, 'flattr_lng'), 'flattrbutton', 'flattrbutton_main');
		add_settings_section('flattrbutton_display', 'Display Settings', array($flattrbutton, 'display_text'), 'flattrbutton');
		add_settings_field('flattrbutton_option_conditions', 'Show button on', array($flattrbutton, 'display_conditions'), 'flattrbutton', 'flattrbutton_display');

		foreach (array('uid', 'cat', 'lng') as $key) {
			register_setting('flattrbutton_options', "flattrbutton_option_$key", array($flattrbutton, "set_$key"));
		}
		register_setting('flattrbutton_options', 'flattrbutton_conditions', array($flattrbutton, 'set_conditions'));
	}
	
	function flattrbutton_add_meta_box () {
		global $flattrbutton;
		add_meta_box('flattr_post_settings', __('Flattr settings'), array($flattrbutton, 'inner_meta_box'), 'post', 'advanced');
	}
endif;
?>
